<?php
namespace AppBundle\Controller;

use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Security;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\HttpFoundation\Request;

use K2Bundle\EntityRepository\TSpecialLunchMenuRepository;
use K2Bundle\EntityRepository\TSpecialLunchMealRepository;
use K2Bundle\EntityRepository\TSpecialLunchOrderRepository;
use K2Bundle\Entity\TSpecialLunchMenu;
use K2Bundle\Entity\TSpecialLunchOrder;
use K2Bundle\Entity\TSpecialLunchMeal;
use AppBundle\Form\LunchOrderType;
use AppBundle\Form\LunchOrderCollectionType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use AppBundle\Entity\LunchOrderCollection;
use AppBundle\Entity\LunchOrderBatch;
use AppBundle\Entity\LunchOrderBatchChild;
use AppBundle\Form\LunchOrderBatchType;
use AppBundle\Form\LunchOrderBatchChildType;
use AppBundle\EntityRepository\OfficerRepository;
use Symfony\Component\HttpFoundation\JsonResponse;
use K2Bundle\Entity\CodeType;
use K2Bundle\EntityRepository\CodeTypeRepository;
use K2Bundle\Entity\Officer;
use K2Bundle\Utilities\Utilities;


class LunchOrderingController extends Controller
{
    /**
     * @Route("/lunch-order-overview", name="lunch_order_overview")
	 * @Security("has_role('ROLE_USER')")
     */

    public function orderOverview(Request $request)
    {
        $orderRep = new TSpecialLunchOrderRepository($this -> get("k2.data"));
        $officerId = Utilities::getValueFromArrayDef('officerId', $this -> get('security.token_storage') -> getToken() -> getUser() -> getUserData());
		$order = $orderRep->getFiltered(array('OfficerID' => $officerId, 'IsInvalidRecord' => 'false'), array('MenuRID.Date') ,  array('officerID','MenuRID', 'Date', 'soupRID.name', 'mainCourseRID.name', 'OrderStatusId.Abbr'));      
        $reversedOrders = array_reverse($order, false);
        return $this -> render('lunchOrdering/Lunch_OrderOverview.html.twig', [
			'result' => $reversedOrders,
        ]);
    }
	
	/**
     * @Route("/lunch-order-available", name="lunch_order_available")
	 * @Security("has_role('ROLE_USER')")
     */

    public function orderAvailable(Request $request)
    {
        $orderRep = new TSpecialLunchOrderRepository($this -> get("k2.data"));
        $officerId = Utilities::getValueFromArrayDef('officerId', $this -> get('security.token_storage') -> getToken() -> getUser() -> getUserData());
		$order = $orderRep->getFiltered(array('OrderStatusId.abbr' => 'Available', 'IsInvalidRecord' => 'false'), array('MenuRID.Date') ,  array('officerID','MenuRID', 'Date', 'soupRID.name', 'mainCourseRID.name', 'OrderStatusId.Abbr'));      
        $reversedOrders = array_reverse($order, false);
        return $this -> render('lunchOrdering/Lunch_OrderAvailable.html.twig', [
			'result' => $reversedOrders,
        ]);
    }

    /**
     * @Route("/cancel-lunch-order/{orderRID}", name="lunch_order_cancel")
	 * @Security("has_role('ROLE_USER')")
     */

    public function orderCancel(Request $request, int $orderRID)
    {
        $orderRep = new TSpecialLunchOrderRepository($this -> get("k2.data"));
		$officerId = Utilities::getValueFromArrayDef('officerId', $this -> get('security.token_storage') -> getToken() -> getUser() -> getUserData());
        $order = $orderRep->getUnique(array('RID'=> $orderRID, 'OfficerID' => $officerId));
        if($order)
		{
			$order->setIsInvalidRecord(true);
			$order->put();
        }
		return $this->redirectToRoute('lunch_order_overview');
    }

    /**
     * @Route("/claim-lunch-order/{orderRID}", name="lunch_order_claim")
	 * @Security("has_role('ROLE_USER')")
     */

    public function orderClaim(Request $request, int $orderRID)
    {
        $orderRep = new TSpecialLunchOrderRepository($this -> get("k2.data"));
		$officerId = Utilities::getValueFromArrayDef('officerId', $this -> get('security.token_storage') -> getToken() -> getUser() -> getUserData());
        $order = $orderRep->getUnique(array('RID'=> $orderRID, 'IsInvalidRecord' => 'false'));
        if($order)
		{
			if($order->getOrderStatusIdOutLinkValue() == 'Available')
            {
				$newStatus = new CodeType($this -> get("k2.data"));
                $newStatus->setCodeTypeId($GLOBALS['NUMFIELD_CONST']['STATUS_ORDERED']); // ordered
                $order->setOrderStatusId($newStatus);
				
				$officerId = Utilities::getValueFromArrayDef('officerId', $this -> get('security.token_storage') -> getToken() -> getUser() -> getUserData());
        
				$newOfficer = new Officer($this -> get("k2.data"));
				$newOfficer->setOfficerID($officerId);
				$order->setOfficerID($newOfficer);
				
				$order->put();
			}
        }
		return $this->redirectToRoute('lunch_order_overview');
    }

    /**
     * @Route("/free-lunch-order/{orderRID}", name="lunch_order_free")
	 * @Security("has_role('ROLE_USER')")
     */

    public function orderFree(Request $request, int $orderRID)
    {
        $orderRep = new TSpecialLunchOrderRepository($this -> get("k2.data"));
		$officerId = Utilities::getValueFromArrayDef('officerId', $this -> get('security.token_storage') -> getToken() -> getUser() -> getUserData());
        $order = $orderRep->getUnique(array('RID'=> $orderRID, 'OfficerID' => $officerId));
        if($order)
        {
            if($order->getOrderStatusIdOutLinkValue() == 'Ordered')
            {
                $newStatus = new CodeType($this -> get("k2.data"));
                $newStatus->setCodeTypeId($GLOBALS['NUMFIELD_CONST']['STATUS_AVAILABLE']); // available
                $order->setOrderStatusId($newStatus);
                $order->put();
            }
        }
//TODO officer id redirectovat zpatky na vlastni overview
        return $this->redirectToRoute('lunch_order_overview');
    }

    /**
     * @Route("/reclaim-lunch-order/{orderRID}", name="lunch_order_reclaim")
	 * @Security("has_role('ROLE_USER')")
     */

    public function orderReclaim(Request $request, int $orderRID)
    {
        $orderRep = new TSpecialLunchOrderRepository($this -> get("k2.data"));
		$officerId = Utilities::getValueFromArrayDef('officerId', $this -> get('security.token_storage') -> getToken() -> getUser() -> getUserData());
        $order = $orderRep->getUnique(array('RID'=> $orderRID, 'OfficerID' => $officerId));
        if($order)
        {
            if($order->getOrderStatusIdOutLinkValue() == 'Available')
            {
                $newStatus = new CodeType($this -> get("k2.data"));
                $newStatus->setCodeTypeId($GLOBALS['NUMFIELD_CONST']['STATUS_ORDERED']); // ordered
                $order->setOrderStatusId($newStatus);
                $order->put();
            }
        }
        return $this->redirectToRoute('lunch_order_overview');
    }

    /**
     * @Route("/new-lunch-order/{year}-{week}", name="new_lunch_order")
	 * @Security("has_role('ROLE_USER')")
     */

     public function newOrder(Request $request, int $year, int $week)
    {    
        $menuRep = new TSpecialLunchMenuRepository($this -> get("k2.data"));  
        $mealRep = new TSpecialLunchMealRepository($this -> get("k2.data"));      
        $orderRep = new TSpecialLunchOrderRepository($this -> get("k2.data"));
        $offRep  = new OfficerRepository($this -> get("k2.data"));    
        $userID = Utilities::getValueFromArrayDef('officerId', $this -> get('security.token_storage') -> getToken() -> getUser() -> getUserData());
        $orders = new LunchOrderBatch();
        $menus = array();
        date_default_timezone_set("Europe/Prague"); 
        $date = date_create();
        $dates = array();
            
        for ($i = 0; $i <= 4; $i++) {
            $date->setISODate($year, $week, $i + 1);
            array_push($dates, date_format($date, 'Y-m-d'));
            $menu = $menuRep->getUnique(array('Date' => date_format($date, 'Y-m-d'), 'StatusId.abbr' => 'Active'), array('RID', 'Date', 'mealChild')); 
            if($menu != null){
              $order = new LunchOrderBatchChild;
              foreach($menu->getMealChild() as &$meal)
              {
                $order->setMenuRID(strval($menu->getRID()));
                $order->setOfficerID($userID);
                $order->setMenuDate(date_format($date, 'Y-m-d'));
                $rid  = $meal->getLunchMealRID()->getRID();
                $desc = strval($meal->getItemNo()) .': ' . strval($meal->getLunchMealRID()->getName());
                $type = strval($meal->getLunchMealRID()->getTypeID());
                $order->addMeal($desc, $rid, $type);  
              }
              $orders->getlunchOrders()->add($order);
            }
        }

        $form = $this->createForm(LunchOrderBatchType::class, $orders);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            $vysledek = $form->getData();

            foreach ($vysledek->getlunchOrders() as &$resOrder) {
                //Pouze pokud je neco zaskrtnuto
                if($resOrder->getSoupRID() || $resOrder->getMainCourseRID())
                {
                    $checkOrder = $orderRep->getUnique(array('OfficerID' => $userID, 'MenuRID' => $resOrder->getMenuRID(), 'IsInvalidRecord' => 'false'), array('MenuRID.Date') ,  array('officerID','MenuRID', 'Date', 'soupRID.name', 'mainCourseRID.name', ' OrderStatusId.abbr'));      
                    if(!$checkOrder)
					{
						$finalOrder = new TSpecialLunchOrder($this -> get("k2.data"));

						$newMenu = new TSpecialLunchMenu($this -> get("k2.data"));
						$newMenu->setRID($resOrder->getMenuRID());               
						$finalOrder->setMenuRID($newMenu); 

						$newOfficer = new Officer($this -> get("k2.data"));
						$newOfficer->setOfficerId($resOrder->getOfficerID());
						$finalOrder->setOfficerId($newOfficer);

						$newStatus = new CodeType($this -> get("k2.data"));
						$newStatus->setCodeTypeId($GLOBALS['NUMFIELD_CONST']['STATUS_NEW']);
						$finalOrder->setOrderStatusId($newStatus);

						if($resOrder->getSoupRID())
						{
						$newSoup = new TSpecialLunchMeal($this -> get("k2.data"));
						$newSoup -> setRID($resOrder->getSoupRID());
						$finalOrder->setSoupRID($newSoup); 
						}
						if($resOrder->getMainCourseRID())
						{
						$newMainCourse = new TSpecialLunchMeal($this -> get("k2.data"));
						$newMainCourse -> setRID($resOrder->getMainCourseRID());
						$finalOrder->setMainCourseRID($newMainCourse); 
						}
						$finalOrder->post();
					}					
                }
            }
            return $this->redirectToRoute('lunch_order_overview');
        }

        return $this -> render('lunchOrdering/Lunch_NewOrder.html.twig', [
            'orders' => $orders,
            'form'   => $form->createView(),
            'formRaw'=> $form,
            'menus'  => $menus,
            'dates'  => $dates,
            'year'   => $year,
            'week'   => $week
        ]);
    }

     /**
     * @Route("/lunch-menu/{year}-{week}", name="lunch_menu")
	 * @Security("has_role('ROLE_USER')")
     */
    public function indexAction(Request $request, int $year, int $week) {
        date_default_timezone_set("Europe/Prague");

        $menuRep = new TSpecialLunchMenuRepository($this -> get("k2.data"));
        $menus = array();

        $date = date_create();
        $dates = array();
   
        for ($i = 0; $i <= 4; $i++) {
            $date->setISODate($year, $week, $i + 1);
            array_push($dates, date_format($date, 'Y-m-d'));

            $menu = $menuRep->getUnique(array('Date' => date_format($date, 'Y-m-d'), 'StatusId.abbr' => 'Active'), array('RID', 'Date', 'mealChild', 'StatusId.abbr')); 
            if($menu != null){
                $menuStatus = $menu->getStatusIdOutLinkValue();
                if($menuStatus == 'Active' || $menuStatus == 'Locked' )
                    array_push($menus, $menu);
            } else { // TODO - nejde mi udelat podminka s operatorem IN
                $menu = $menuRep->getUnique(array('Date' => date_format($date, 'Y-m-d'), 'StatusId.abbr' => 'Locked'), array('RID', 'Date', 'mealChild', 'StatusId.abbr')); 
                if($menu != null){
                    $menuStatus = $menu->getStatusIdOutLinkValue();
                    if($menuStatus == 'Active' || $menuStatus == 'Locked' )
                        array_push($menus, $menu);         
                }      
            }
        }
            
        return $this -> render('lunchOrdering/lunch_Menu.html.twig', [
            'menus' => $menus,
            'dates' => $dates,
            'currentYear' => $year,
            'currentWeek' => $week
        ]);
    }
}
